'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var typescript = require('typescript');
var fs = _interopDefault(require('fs'));
var path = _interopDefault(require('path'));

var tsPathsResolve = function (_a) {
    var _b = _a === void 0 ? {} : _a, _c = _b.tsConfigPath, tsConfigPath = _c === void 0 ? process.env["TS_NODE_PROJECT"] || typescript.findConfigFile(".", typescript.sys.fileExists) || "tsconfig.json" : _c, _d = _b.logLevel, logLevel = _d === void 0 ? "warn" : _d;
    var pluginName = "ts-paths";
    var compilerOptions = getTsConfig(tsConfigPath).compilerOptions;
    var baseUrl = path.resolve(path.dirname(tsConfigPath), compilerOptions.baseUrl);
    var mappings = createMappings(compilerOptions, pluginName, logLevel);
    var host = typescript.createCompilerHost(compilerOptions);
    return {
        name: pluginName,
        resolveId: function (source, importer) {
            if (typeof importer === "undefined" || source.startsWith("\0") || mappings.length == 0) {
                return null;
            }
            for (var _i = 0, mappings_1 = mappings; _i < mappings_1.length; _i++) {
                var mapping = mappings_1[_i];
                var resolved = findMapping({
                    mapping: mapping,
                    source: source,
                    importer: importer,
                    baseUrl: baseUrl,
                    compilerOptions: compilerOptions,
                    host: host,
                });
                if (resolved) {
                    if (logLevel === "debug") {
                        console.log("\u001B[36m[" + pluginName + "]\u001B[0m", source, "->", resolved);
                    }
                    return resolved;
                }
            }
            return null;
        },
    };
};
var getTsConfig = function (configPath) {
    var _a = typescript.readConfigFile(configPath, typescript.sys.readFile), config = _a.config, error = _a.error;
    if (error) {
        throw new Error(error.messageText.toString());
    }
    var compilerOptions = config.compilerOptions;
    compilerOptions = compilerOptions || {};
    compilerOptions.baseUrl = compilerOptions.baseUrl || ".";
    switch (String.prototype.toLocaleLowerCase.call(compilerOptions.moduleResolution)) {
        case "classic":
            compilerOptions.moduleResolution = typescript.ModuleResolutionKind.Classic;
            break;
        default:
            compilerOptions.moduleResolution = typescript.ModuleResolutionKind.NodeJs;
            break;
    }
    return { compilerOptions: compilerOptions };
};
var createMappings = function (compilerOptions, pluginName, logLevel) {
    var escapeRegExp = function (str) { return str.replace(/[-\/\\^$*+?\.()[\]{}]/g, "\\$&"); };
    var mappings = [];
    var paths = compilerOptions.paths || {};
    if (logLevel !== "none" && Object.keys(paths).length === 0) {
        console.log("\u001B[1;33m(!) [" + pluginName + "]: typescript path alias are empty.\u001B[0m");
    }
    for (var _i = 0, _a = Object.keys(paths); _i < _a.length; _i++) {
        var alias = _a[_i];
        var wildcard = alias.indexOf("*") !== -1;
        var targets = paths[alias].filter(function (target) {
            if (target.startsWith("@types") || target.endsWith(".d.ts")) {
                if (logLevel === "debug") {
                    console.log("\u001B[1;33m(!) [" + pluginName + "]: type defined " + target + " is ignored.\u001B[0m");
                }
                return false;
            }
            return true;
        });
        if (alias === "*") {
            mappings.push({ alias: { source: alias, wildcard: wildcard, pattern: /(.*)/ }, targets: targets });
            continue;
        }
        var excapedAlias = escapeRegExp(alias);
        var pattern = wildcard
            ? new RegExp("^" + excapedAlias.replace("\\*", "(.*)"))
            : new RegExp("^" + excapedAlias + "$");
        mappings.push({ alias: { source: alias, wildcard: wildcard, pattern: pattern }, targets: targets });
    }
    if (logLevel === "debug") {
        for (var _b = 0, mappings_2 = mappings; _b < mappings_2.length; _b++) {
            var mapping = mappings_2[_b];
            console.log("\u001B[36m[" + pluginName + "]\u001B[0m", "pattern:", mapping.alias.pattern, "targets:", mapping.targets);
        }
    }
    return mappings;
};
var findMapping = function (_a) {
    var mapping = _a.mapping, source = _a.source, importer = _a.importer, baseUrl = _a.baseUrl, compilerOptions = _a.compilerOptions, host = _a.host;
    var match = source.match(mapping.alias.pattern);
    if (!match) {
        return "";
    }
    for (var _i = 0, _b = mapping.targets; _i < _b.length; _i++) {
        var target = _b[_i];
        var predicted = target;
        if (mapping.alias.wildcard) {
            predicted = target.replace("*", match[1]);
        }
        var answer = path.resolve(baseUrl, predicted);
        if (answer.indexOf("node_modules/") != -1) {
            return answer;
        }
        var resolvedModule = typescript.resolveModuleName(answer, importer, compilerOptions, host).resolvedModule;
        if (resolvedModule) {
            return resolvedModule.resolvedFileName;
        }
        if (fs.existsSync(answer)) {
            return answer;
        }
    }
    return "";
};

exports.default = tsPathsResolve;
exports.tsPathsResolve = tsPathsResolve;
//# sourceMappingURL=index.cjs.js.map
